@@ -38,6 +38,10 @@
 #include "giggle-history.h"
 #include "eggfindbar.h"
 
+#ifdef GDK_WINDOWING_QUARTZ
+#include "ige-mac-menu.h"
+#endif
+
 typedef struct GiggleWindowPriv GiggleWindowPriv;
 
 struct GiggleWindowPriv {
@@ -64,6 +68,11 @@ struct GiggleWindowPriv {
 	GtkWidget           *diff_current_window;
 
 	GiggleConfiguration *configuration;
+
+	gint                 width;
+	gint                 height;
+	gint                 x;
+	gint                 y;
 };
 
 enum {
@@ -73,6 +82,8 @@ enum {
 
 
 static void window_finalize                       (GObject           *object);
+static gboolean window_configure_event            (GtkWidget           *widget,
+						   GdkEventConfigure   *event);
 
 static void window_add_widget_cb                  (GtkUIManager      *merge,
 						   GtkWidget         *widget,
@@ -106,6 +117,8 @@ static void window_action_history_go_back         (GtkAction         *action,
 						   GiggleWindow      *window);
 static void window_action_history_go_forward      (GtkAction         *action,
 						   GiggleWindow      *window);
+static void window_action_history_refresh         (GtkAction         *action,
+						   GiggleWindow      *window);
 static void window_directory_changed_cb           (GiggleGit         *git,
 						   GParamSpec        *arg,
 						   GiggleWindow      *window);
@@ -184,7 +197,7 @@ static const GtkActionEntry action_entries[] = {
 	  G_CALLBACK (window_action_personal_details_cb)
 	},
 	{ "Find", GTK_STOCK_FIND,
-	  N_("_Find..."), NULL, N_("Find..."),
+	  N_("_Find..."), "slash", N_("Find..."),
 	  G_CALLBACK (window_action_find_cb)
 	},
 	{ "FindNext", NULL,
@@ -209,6 +222,10 @@ static const GtkActionEntry action_entries[] = {
 	  N_("_Forward"), "<alt>Right", NULL,
 	  G_CALLBACK (window_action_history_go_forward)
 	},
+	{ "RefreshHistory", GTK_STOCK_REFRESH,
+	  N_("_Refresh"), "F5", NULL,
+	  G_CALLBACK (window_action_history_refresh)
+	},
 };
 
 static const gchar *ui_layout =
@@ -236,6 +253,8 @@ static const gchar *ui_layout =
 	"      <menuitem action='CompactMode'/>"
 	"      <menuitem action='ViewFileList'/>"
 	"      <menuitem action='ViewGraph'/>"
+	"      <separator/>"
+	"      <menuitem action='RefreshHistory'/>"
 	"    </menu>"
 	"    <menu action='GoMenu'>"
 	"      <menuitem action='BackHistory'/>"
@@ -248,6 +267,7 @@ static const gchar *ui_layout =
 	"  <toolbar name='MainToolbar'>"
 	"    <toolitem action='BackHistory'/>"
 	"    <toolitem action='ForwardHistory'/>"
+	"    <toolitem action='RefreshHistory'/>"
 	"  </toolbar>"
 	"</ui>";
 
@@ -268,8 +288,10 @@ static void
 giggle_window_class_init (GiggleWindowClass *class)
 {
 	GObjectClass   *object_class = G_OBJECT_CLASS (class);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
 
 	object_class->finalize = window_finalize;
+	widget_class->configure_event = window_configure_event;
 
 	g_type_class_add_private (object_class, sizeof (GiggleWindowPriv));
 }
@@ -321,6 +343,30 @@ window_create_menu (GiggleWindow *window)
 				  G_CALLBACK (window_recent_repositories_update), window);
 
 	window_recent_repositories_update (window);
+
+#ifdef GDK_WINDOWING_QUARTZ
+	{
+		GtkWidget       *menu;
+		GtkWidget       *item;
+		IgeMacMenuGroup *group;
+
+		menu = gtk_ui_manager_get_widget (priv->ui_manager,
+						  "/MainMenubar");
+
+		ige_mac_menu_set_menu_bar (GTK_MENU_SHELL (menu));
+		gtk_widget_hide (menu);
+
+		item = gtk_ui_manager_get_widget (priv->ui_manager,
+						  "/MainMenubar/ProjectMenu/Quit");
+		ige_mac_menu_set_quit_menu_item (GTK_MENU_ITEM (item));
+
+		item = gtk_ui_manager_get_widget (priv->ui_manager,
+						  "/MainMenubar/HelpMenu/About");
+		group =  ige_mac_menu_add_app_menu_group ();
+		ige_mac_menu_add_app_menu_item  (group, GTK_MENU_ITEM (item),
+						 _("About Giggle"));
+	}
+#endif
 }
 
 static void
@@ -388,26 +434,28 @@ window_configuration_committed (GiggleConfiguration *configuration,
 }
 
 static void
-save_state_and_exit (GiggleWindow *window)
+save_state (GiggleWindow *window)
 {
 	GiggleWindowPriv *priv;
 	gboolean          compact;
-	gint              w, h, x, y;
 	gchar             buf[25];
+	gboolean          maximized;
 
 	priv = GET_PRIV (window);
 
-	gtk_window_get_size (GTK_WINDOW (window), &w, &h);
-	gtk_window_get_position (GTK_WINDOW (window), &x, &y);
-
-	g_snprintf (buf, sizeof (buf), "%dx%d+%d+%d", w, h, x, y);
+	g_snprintf (buf, sizeof (buf), "%dx%d+%d+%d", priv->width, priv->height, priv->x, priv->y);
 	giggle_configuration_set_field (priv->configuration,
 					CONFIG_FIELD_MAIN_WINDOW_GEOMETRY, buf);
 
+	maximized = gdk_window_get_state (GTK_WIDGET (window)->window) & GDK_WINDOW_STATE_MAXIMIZED;
+	giggle_configuration_set_boolean_field (priv->configuration,
+						CONFIG_FIELD_MAIN_WINDOW_MAXIMIZED,
+						maximized);
+
 	compact = giggle_view_history_get_compact_mode (GIGGLE_VIEW_HISTORY (priv->history_view));
-	giggle_configuration_set_field (priv->configuration,
-					CONFIG_FIELD_COMPACT_MODE,
-					compact ? "true" : "false");
+	giggle_configuration_set_boolean_field (priv->configuration,
+						CONFIG_FIELD_COMPACT_MODE,
+						compact);
 
 	giggle_configuration_commit (priv->configuration,
 				     window_configuration_committed,
@@ -419,7 +467,7 @@ window_delete_event_cb (GtkWidget *widget,
 			GdkEvent  *event,
 			gpointer   user_data)
 {
-	save_state_and_exit (GIGGLE_WINDOW (widget));
+	save_state (GIGGLE_WINDOW (widget));
 	gtk_widget_hide (widget);
 	return TRUE;
 }
@@ -429,17 +477,14 @@ window_bind_state (GiggleWindow *window)
 {
 	GiggleWindowPriv *priv;
 	GtkAction *action;
-	const char *geom_str, *compact_str;
-	gboolean compact = FALSE, found_geometry = FALSE;
+	const char *geom_str;
+	gboolean compact, found_geometry = FALSE;
 
 	priv = GET_PRIV (window);
 	g_assert (priv->configuration);
 
-	compact_str = giggle_configuration_get_field (priv->configuration, CONFIG_FIELD_COMPACT_MODE);
-	if (compact_str) {
-		compact = strcmp (compact_str, "true") == 0;
-	}
 	/* set the toggle menu item to indicate the current compact setting */
+	compact = giggle_configuration_get_boolean_field (priv->configuration, CONFIG_FIELD_COMPACT_MODE);
 	action = gtk_ui_manager_get_action (priv->ui_manager, COMPACT_MODE);
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), compact);
 	giggle_view_history_set_compact_mode (GIGGLE_VIEW_HISTORY (priv->history_view), compact);
@@ -453,7 +498,14 @@ window_bind_state (GiggleWindow *window)
 		gtk_window_set_default_size (GTK_WINDOW (window), 700, 550);
 	}
 
+	if (giggle_configuration_get_boolean_field (priv->configuration, CONFIG_FIELD_MAIN_WINDOW_MAXIMIZED)) {
+		gtk_window_maximize (GTK_WINDOW (window));
+	}
+
 	gtk_widget_show (GTK_WIDGET (window));
+	if (priv->diff_current_window) {
+		gtk_widget_show (priv->diff_current_window);
+	}
 
 	/* set up a callback to save the new UI state on application exit */
 	g_signal_connect (GTK_WINDOW (window), "delete-event",
@@ -543,7 +595,7 @@ window_finalize (GObject *object)
 	GiggleWindowPriv *priv;
 
 	priv = GET_PRIV (object);
-	
+
 	g_object_unref (priv->ui_manager);
 	g_object_unref (priv->git);
 	g_object_unref (priv->recent_manager);
@@ -553,6 +605,20 @@ window_finalize (GObject *object)
 	G_OBJECT_CLASS (giggle_window_parent_class)->finalize (object);
 }
 
+static gboolean
+window_configure_event (GtkWidget         *widget,
+			GdkEventConfigure *event)
+{
+	GiggleWindowPriv *priv = GET_PRIV (widget);
+
+	if (! (gdk_window_get_state (widget->window) & GDK_WINDOW_STATE_MAXIMIZED)) {
+		gtk_window_get_size (GTK_WINDOW (widget), &priv->width, &priv->height);
+		gtk_window_get_position (GTK_WINDOW (widget), &priv->x, &priv->y);
+	}
+
+	return GTK_WIDGET_CLASS (giggle_window_parent_class)->configure_event (widget, event);;
+}
+
 static void
 window_recent_repositories_add (GiggleWindow *window)
 {
@@ -714,7 +780,7 @@ static void
 window_action_quit_cb (GtkAction    *action,
 		       GiggleWindow *window)
 {
-	save_state_and_exit (window);
+	save_state (window);
 	gtk_widget_hide (GTK_WIDGET (window));
 }
 
@@ -798,22 +864,7 @@ static void
 window_action_diff_cb (GtkAction    *action,
 		       GiggleWindow *window)
 {
-	GiggleWindowPriv *priv;
-
-	priv = GET_PRIV (window);
-
-	if (!priv->diff_current_window) {
-		priv->diff_current_window = giggle_diff_window_new ();
- 
-		gtk_window_set_transient_for (GTK_WINDOW (priv->diff_current_window),
-					      GTK_WINDOW (window));
-		g_signal_connect (priv->diff_current_window, "delete-event",
-				  G_CALLBACK (gtk_widget_hide_on_delete), NULL);
-		g_signal_connect_after (priv->diff_current_window, "response",
-					G_CALLBACK (gtk_widget_hide), NULL);
-	}
-
-	gtk_widget_show (priv->diff_current_window);
+	giggle_window_show_diff_window (window);
 }
 
 static void
@@ -1006,6 +1057,7 @@ window_action_about_cb (GtkAction    *action,
 			       "name", "Giggle",
 			       "copyright", "Copyright \xc2\xa9 2007 Imendio AB",
 			       "translator-credits", _("translator-credits"),
+			       "logo-icon-name", PACKAGE,
 			       "version", VERSION,
 			       "authors", authors,
 			       NULL);
@@ -1084,6 +1136,18 @@ window_action_history_go_forward (GtkAction    *action,
 	giggle_history_go_forward (GIGGLE_HISTORY (page));
 }
 
+static void
+window_action_history_refresh (GtkAction    *action,
+			       GiggleWindow *window)
+{
+	GiggleWindowPriv *priv;
+	const gchar      *directory;
+
+	priv = GET_PRIV (window);
+	directory = giggle_git_get_directory (priv->git);
+	giggle_window_set_directory (window, directory);
+}
+
 GtkWidget *
 giggle_window_new (void)
 {
@@ -1102,3 +1166,26 @@ giggle_window_get_git (GiggleWindow *self)
 	return GET_PRIV (self)->git;
 }
 
+void
+giggle_window_show_diff_window (GiggleWindow *self)
+{
+	GiggleWindowPriv *priv;
+
+	priv = GET_PRIV (self);
+
+	if (!priv->diff_current_window) {
+		priv->diff_current_window = giggle_diff_window_new ();
+
+		gtk_window_set_transient_for (GTK_WINDOW (priv->diff_current_window),
+					      GTK_WINDOW (self));
+		g_signal_connect (priv->diff_current_window, "delete-event",
+				  G_CALLBACK (gtk_widget_hide_on_delete), NULL);
+		g_signal_connect_after (priv->diff_current_window, "response",
+					G_CALLBACK (gtk_widget_hide), NULL);
+	}
+
+	if (GTK_WIDGET_REALIZED (self)) {
+		gtk_widget_show (priv->diff_current_window);
+	}
+}
+
